@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999-2002, 2007-2014, 2019-2021 Free Software Foundation,
+    Copyright (C) 1999-2002, 2007-2014, 2019-2022 Free Software Foundation,
     Inc.
 
     This program is free software; you can redistribute it and/or modify
@@ -27,6 +27,12 @@
 #ifndef PED_DISK_H_INCLUDED
 #define PED_DISK_H_INCLUDED
 
+/* Include these to work around gnulib redefining free, read, etc. which causes problems
+ * with pt-common.h
+ */
+#include <stdlib.h>
+#include <unistd.h>
+
 /**
  * Disk flags
  */
@@ -78,10 +84,11 @@ enum _PedPartitionFlag {
         PED_PARTITION_IRST=17,
         PED_PARTITION_ESP=18,
         PED_PARTITION_CHROMEOS_KERNEL=19,
-        PED_PARTITION_BLS_BOOT=20
+        PED_PARTITION_BLS_BOOT=20,
+        PED_PARTITION_LINUX_HOME=21,
 };
 #define PED_PARTITION_FIRST_FLAG        PED_PARTITION_BOOT
-#define PED_PARTITION_LAST_FLAG         PED_PARTITION_BLS_BOOT
+#define PED_PARTITION_LAST_FLAG         PED_PARTITION_LINUX_HOME
 
 enum _PedDiskTypeFeature {
         PED_DISK_TYPE_EXTENDED=1,       /**< supports extended partitions */
