@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999-2000, 2007, 2009-2014, 2019-2021 Free Software
+    Copyright (C) 1999-2000, 2007, 2009-2014, 2019-2022 Free Software
     Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
@@ -46,6 +46,7 @@ typedef enum _PedExceptionType PedExceptionType;
  * Option for resolving the exception
  */
 enum _PedExceptionOption {
+	/* individual options */
 	PED_EXCEPTION_UNHANDLED=0,
 	PED_EXCEPTION_FIX=1,
 	PED_EXCEPTION_YES=2,
@@ -54,19 +55,23 @@ enum _PedExceptionOption {
 	PED_EXCEPTION_RETRY=16,
 	PED_EXCEPTION_IGNORE=32,
 	PED_EXCEPTION_CANCEL=64,
+
+	/* combinations of individual options */
+	PED_EXCEPTION_OK_CANCEL = PED_EXCEPTION_OK + PED_EXCEPTION_CANCEL,
+	PED_EXCEPTION_YES_NO = PED_EXCEPTION_YES + PED_EXCEPTION_NO,
+	PED_EXCEPTION_YES_NO_CANCEL =
+		PED_EXCEPTION_YES_NO + PED_EXCEPTION_CANCEL,
+	PED_EXCEPTION_IGNORE_CANCEL =
+		PED_EXCEPTION_IGNORE + PED_EXCEPTION_CANCEL,
+	PED_EXCEPTION_RETRY_CANCEL = PED_EXCEPTION_RETRY + PED_EXCEPTION_CANCEL,
+	PED_EXCEPTION_RETRY_IGNORE_CANCEL =
+		PED_EXCEPTION_RETRY + PED_EXCEPTION_IGNORE_CANCEL,
 };
-typedef enum _PedExceptionOption PedExceptionOption;
-#define PED_EXCEPTION_OK_CANCEL	    (PED_EXCEPTION_OK + PED_EXCEPTION_CANCEL)
-#define PED_EXCEPTION_YES_NO	    (PED_EXCEPTION_YES + PED_EXCEPTION_NO)
-#define PED_EXCEPTION_YES_NO_CANCEL (PED_EXCEPTION_YES_NO \
-				     + PED_EXCEPTION_CANCEL)
-#define PED_EXCEPTION_IGNORE_CANCEL (PED_EXCEPTION_IGNORE \
-				     + PED_EXCEPTION_CANCEL)
-#define PED_EXCEPTION_RETRY_CANCEL  (PED_EXCEPTION_RETRY + PED_EXCEPTION_CANCEL)
-#define PED_EXCEPTION_RETRY_IGNORE_CANCEL (PED_EXCEPTION_RETRY \
-					   + PED_EXCEPTION_IGNORE_CANCEL)
+
 #define PED_EXCEPTION_OPTION_FIRST PED_EXCEPTION_FIX
-#define PED_EXCEPTION_OPTION_LAST PED_EXCEPTION_CANCEL
+#define PED_EXCEPTION_OPTION_LAST PED_EXCEPTION_CANCEL	/* last individual option */
+
+typedef enum _PedExceptionOption PedExceptionOption;
 
 /**
  * Structure with information about exception
