@@ -7,9 +7,23 @@
 #include <check.h>
 
 #include "common.h"
+#include "xstrtol.h"
 
 #define STREQ(a, b) (strcmp (a, b) == 0)
 
+size_t get_sector_size (void)
+{
+  char *p = getenv ("PARTED_SECTOR_SIZE");
+  size_t ss = 512;
+  unsigned long val;
+  if (p
+      && xstrtoul (p, NULL, 10, &val, NULL) == LONGINT_OK
+      && val % 512 == 0)
+    ss = val;
+
+  return ss;
+}
+
 PedExceptionOption
 _test_exception_handler (PedException* e)
 {
@@ -69,6 +83,9 @@ _implemented_disk_label (const char *label)
         /* FIXME: these have minor problems, so skip them, temporarily.  */
         if (STREQ (label, "amiga")) return 0;
 
+        if (STREQ (label, "atari") && get_sector_size() != 512)
+                return 0;
+
         /* Not implemented yet */
         if (STREQ (label, "aix")) return 0;
         if (STREQ (label, "pc98")) return 0;
